#ifndef DISPATCH_GOPS_H
#define DISPATCH_GOPS_H

#include <dol.h>
#include "global.h"

#ifdef INPUT_FILE_QUERY
char input_filename[50];
#else
extern char input_filename[50];
#endif

#define PORT_OUT "out"

//local variables
typedef struct _local_states {
    char id[64];
    char name[15];    
    FILE *infile;
    int System_Stream_Flag;  /**< Indicates if stream must be demuxed. */
    int gop_count;
    int gop_port_count;      /**< Current output port number. */
    int prev_gop_port;       /**< Previous output port number. */
    bool attach_b_pics;      /**< Indicate if B-pics must be attached.
                                  This is needed for open GOPs only */
    ReadBuffer read_buffer;
    GopDispatchedDatatype gop_dispatched_datatype;
    QuantizerMatrices quantizer_matrices;
    unsigned char send_buffer[MAX_GOP_PIC_BUFFER_SIZE];
} Dispatch_gops_State;   /**< Local Variables of the Dispatch GOP process */

/* mandatory function prototypes */
void dispatch_gops_init(DOLProcess *);
int dispatch_gops_fire(DOLProcess *);

/* DOL process declaration */
extern DOLProcess dispatch_gops;

/* additional function prototypes */
void dispatch_gops_finish(DOLProcess *);
void dispatch_gop(DOLProcess *);
int get_videosequence_header(DOLProcess *);
void initialize_sequence(DOLProcess *);
int get_videosequence_extension(DOLProcess *);
void get_seq_scal_ext(DOLProcess *);
void Initialize_Buffer(DOLProcess *);
void Fill_Buffer(DOLProcess *);
void Flush_Buffer(DOLProcess *, int);
void Next_Packet(DOLProcess *);
void next_start_code(DOLProcess *);
int Get_Long(DOLProcess *);
int Get_Word(DOLProcess *);
int Get_Byte(DOLProcess *);
unsigned int Show_Bits(DOLProcess *, int);
void Flush_Buffer32(DOLProcess *);
unsigned int Get_Bits32(DOLProcess *);
unsigned int Get_Bits(DOLProcess *p, int);
void Error(DOLProcess *, char *);

/* define scan array */
unsigned char scan[2][64] =
{
  { /* Zig-Zag scan pattern  */
    0,1,8,16,9,2,3,10,17,24,32,25,18,11,4,5,
    12,19,26,33,40,48,41,34,27,20,13,6,7,14,21,28,
    35,42,49,56,57,50,43,36,29,22,15,23,30,37,44,51,
    58,59,52,45,38,31,39,46,53,60,61,54,47,55,62,63
  },
  { /* Alternate scan pattern */
    0,8,16,24,1,9,2,10,17,25,32,40,48,56,57,49,
    41,33,26,18,3,11,4,12,19,27,34,42,50,58,35,43,
    51,59,20,28,5,13,6,14,21,29,36,44,52,60,37,45,
    53,61,22,30,7,15,23,31,38,46,54,62,39,47,55,63
  }
};


/* define the default quantizer matrix */
unsigned char default_intra_quantizer_matrix[64] =
{
  8, 16, 19, 22, 26, 27, 29, 34,
  16, 16, 22, 24, 27, 29, 34, 37,
  19, 22, 26, 27, 29, 34, 34, 38,
  22, 22, 26, 27, 29, 34, 37, 40,
  22, 26, 27, 29, 32, 35, 40, 48,
  26, 27, 29, 32, 35, 40, 48, 58,
  26, 27, 29, 34, 38, 46, 56, 69,
  27, 29, 35, 38, 46, 56, 69, 83
};


#endif
