#ifndef DISPATCH_MB_H
#define DISPATCH_MB_H

#include <dol.h>
#include "global.h"

#define PORT_OUT "out"
#define PORT_IN "in"

//local variables
typedef struct _local_states {
    char id[64];
    char name[15];
    BitBuffer bit_buffer;   /**< Buffer that contains the received GOP */
    MbDispatchedDatatype mb_dispatched_datatype;
    GopDispatchedDatatype gop_dispatched_datatype;
    QuantizerMatrices quantizer_matrices;    
    int send_buffer[MAX_MB_BUFFER_SIZE];
    int send_buffer_size;
    int send_buffer_pos;
    int dc_dct_predictor[3];
    int prev_mba;           /**< Address of the previous MB */
    int mb_address_max;     /**< Address of the last MB of a picture */
    bool first_mb_of_slice;
    int out_port_count;
    int gop_count;
#ifdef DEBUG
    int pic_count;
    int slice_count;
    int mb_count;
#endif
    bool full_pel_forward_vector;
    char forward_f_code;
    bool full_pel_backward_vector;
    char backward_f_code;
    char f_code[2][2];
    char intra_dc_precision;
    bool frame_pred_frame_dct;
    bool concealment_motion_vectors;
    bool q_scale_type;
    bool intra_vlc_format;
    bool alternate_scan;
    int quantizer_scale_code;
    int quantizer_scale; /* depends on q_scale_type and quantizer_scale_code */
    int mb_address_increment;/* Indicates the number of skipped macroblocks */
    int motion_vector_count;
    int mv_format;
    int dmv;
    int coded_block_pattern;  /* Indicates which blocks of the mb are coded */
    int mb_row;               /* Row of the current mb in the picture */
    int stwclass;    
} Dispatch_mb_State;          /* Lokal variables of the Dispatch mb process */

/* mandatory function prototypes */
void dispatch_mb_init(DOLProcess *);
int dispatch_mb_fire(DOLProcess *);

/* dol process declaration */
extern DOLProcess dispatch_mb;

/* additional function prototypes */
void dispatch_mb_finish(DOLProcess *);
void parse_picture(DOLProcess *);
void parse_slice(DOLProcess *);
void parse_mb(DOLProcess *);
void macroblock_modes(DOLProcess *);
void get_mvs(DOLProcess *);
void coded_block_patterns(DOLProcess *);
void fill_send_buffer(DOLProcess *);
void send_mb(DOLProcess *);
unsigned int bb_Show_Bits(DOLProcess *, int);
void bb_Advance_Bits(DOLProcess *, int);
unsigned int bb_Get_Bits(DOLProcess *, int);
int bb_next_start_code(DOLProcess *);
int bb_Byte_Align(DOLProcess *);
int check_for_slice(int);

/* additional extern function prototypes (defined elsewere) */
extern void Error(DOLProcess *, char *);

/* externally defined matrices */
extern unsigned char scan[2][64];

/* define the nonlinear quantizer scale array */
unsigned char Non_Linear_quantizer_scale[32] =
{
   0, 1, 2, 3, 4, 5, 6, 7,
   8,10,12,14,16,18,20,22,
  24,28,32,36,40,44,48,52,
  56,64,72,80,88,96,104,112
};

#endif
