/** @file
    Routines for motion vector decoding.
*/

/* decode motion vector component
   NOTE: different arithmetic used than in 7.6.3.1. */
static void decode_motion_vector(int *prediction, int r_size, int motion_code, int motion_residual,int full_pel_vector)
{
  int lim, vec;

  lim = 16<<r_size;
  vec = full_pel_vector ? (*prediction >> 1) : (*prediction);

  if (motion_code>0)
  {
    vec+= ((motion_code-1)<<r_size) + motion_residual + 1;
    if (vec>=lim)
      vec-= lim + lim;
  }
  else if (motion_code<0)
  {
    vec-= ((-motion_code-1)<<r_size) + motion_residual + 1;
    if (vec<-lim)
      vec+= lim + lim;
  }
  *prediction = full_pel_vector ? (vec<<1) : vec;
}

/* get and decode motion vector and differential motion vector 
   for one prediction */
void motion_vector(DOLProcess *p, int *PMV, int *dmvector, int h_r_size, int v_r_size, int dmv, int mvscale, int full_pel_vector)
{
  int motion_code, motion_residual;

  /* horizontal component */
  motion_code = vld_motion_code(p);

  motion_residual = (h_r_size!=0 && motion_code!=0) ? 
                     bb_Get_Bits(p, h_r_size) : 0;

  decode_motion_vector(&PMV[0], h_r_size, motion_code, motion_residual, full_pel_vector);

  if (dmv)
    dmvector[0] = vld_dmvector(p);

  /* vertical component */
  motion_code = vld_motion_code(p);
  motion_residual = (v_r_size!=0 && motion_code!=0) ? 
                     bb_Get_Bits(p, v_r_size) : 0;

  if (mvscale)
    PMV[1] >>= 1;

  decode_motion_vector(&PMV[1], v_r_size, motion_code, motion_residual, full_pel_vector);

  if (mvscale)
    PMV[1] <<= 1;

  if (dmv)
    dmvector[1] = vld_dmvector(p);
}

void motion_vectors(DOLProcess *p, int s, int h_r_size, int v_r_size, int mvscale)
{
    MbDispatchedDatatype *mdd = &(p->local->mb_dispatched_datatype);

    if (p->local->motion_vector_count==1) {
        if (p->local->mv_format==MV_FIELD && !(p->local->dmv)) {
            mdd->motion_vertical_field_select[1][s] = 
             mdd->motion_vertical_field_select[0][s] = bb_Get_Bits(p, 1);
        }
        
        motion_vector(p, mdd->PMV[0][s], mdd->dmvector, h_r_size, v_r_size, p->local->dmv, mvscale, 0);

        /* update other motion vector predictors */
        mdd->PMV[1][s][0] = mdd->PMV[0][s][0];
        mdd->PMV[1][s][1] = mdd->PMV[0][s][1];
    }
    else {
        mdd->motion_vertical_field_select[0][s] = bb_Get_Bits(p, 1);

        motion_vector(p, mdd->PMV[0][s], mdd->dmvector, h_r_size, v_r_size, p->local->dmv, mvscale, 0);

        mdd->motion_vertical_field_select[1][s] = bb_Get_Bits(p, 1);

        motion_vector(p, mdd->PMV[1][s], mdd->dmvector, h_r_size, v_r_size, p->local->dmv, mvscale, 0);
    }
}
